import {
    world,
    BlockLocation,
    Location,
    EntityQueryOptions
} from "@minecraft/server"
import {
    ActionFormData,
    ModalFormData
} from "@minecraft/server-ui"

export function editEnderAltar(entity, source) {
    const form = new ModalFormData()
    form.title('title.edit_ender_altar')
    form.textField("text.enter_ender_altar_name", "", `${entity.nameTag}`)
    form.textField("text.enter_ender_altar_description", "", `${entity.getTags().find(tag => tag.startsWith('about_ender_altar:'))?.replace('about_ender_altar:', '')}`)
    if (entity.hasTag('is_global')) {
        form.toggle('toggle.is_global', true)
    } else if (entity.hasTag('not_global')) {
        form.toggle('toggle.is_global', false)
    }
    form.show(source).then((r) => {
        const { canceled, formValues } = r
        //TextField (Name of the ender_altar)
        if (formValues[0] != '') {
            try {
                entity.nameTag = `${formValues[0]}`
            } catch { }
        }
        if (formValues[1] != `${entity.getTags().find(tag => tag.startsWith('about_ender_altar:'))?.replace('about_ender_altar:', '')}`) {
            try {
                entity.removeTag(`${entity.getTags().find(tag => tag.startsWith('about_ender_altar:'))}`)
                entity.addTag(`about_ender_altar:${formValues[1]}`)
            } catch { }
        }
        if (formValues[2] == true) {
            try {
                entity.removeTag(`not_global`)
                entity.addTag(`is_global`)
            } catch { }
        } else {
            entity.removeTag(`is_global`)
            entity.addTag(`not_global`)
        }
    })
}

export function ender_altarScreen(source, playerDimension, playerLevel, x, y, z) {
    let form_responses = []
    const form = new ActionFormData()
    //Buttons and Fake Buttons
    Array.from(world.getDimension(`${playerDimension}`).getEntities({
        type: "way:ender_altar_dummy",
        location: new Location(x, y - 0.5, z),
        maxDistance: 1
    })).forEach(entity => {
        if (entity.id == `${entity.getTags().find((tag) => tag.startsWith("entity_id:")).replaceAll("entity_id:", "")}`) {
            form.title(`id.ender_altar_screen`)
            form.body(`${source.getTags().find((tag) => tag.startsWith("dimensions_tabs:"))}`)
            {
                if (source.getTags().find((tag) => tag.startsWith("dimensions_tabs:overworld"))) {
                    form.button(`button.overworld_locked`)
                } else { form.button(`button.overworld`) }
                form_responses.push(r => {
                    source.addTag("dimensions_tabs:overworld")
                    source.removeTag("dimensions_tabs:nether")
                    source.removeTag("dimensions_tabs:the_end")
                    ender_altarScreen(source, playerDimension, playerLevel, x, y, z)
                })
            }
            {
                if (source.getTags().find((tag) => tag.startsWith("dimensions_tabs:nether"))) {
                    form.button(`button.nether_locked`)
                } else { form.button(`button.nether`) }
                form_responses.push(r => {
                    source.addTag("dimensions_tabs:nether")
                    source.removeTag("dimensions_tabs:overworld")
                    source.removeTag("dimensions_tabs:the_end")
                    ender_altarScreen(source, playerDimension, playerLevel, x, y, z)
                })
            }
            {
                if (source.getTags().find((tag) => tag.startsWith("dimensions_tabs:the_end"))) {
                    form.button(`button.the_end_locked`)
                } else { form.button(`button.the_end`) }
                form_responses.push(r => {
                    source.addTag("dimensions_tabs:the_end")
                    source.removeTag("dimensions_tabs:nether")
                    source.removeTag("dimensions_tabs:overworld")
                    ender_altarScreen(source, playerDimension, playerLevel, x, y, z)
                })
            }
            {
                if (!entity.getTags().find((tag) => tag.startsWith(`ender_altar_owner:${source.nameTag}`))) {
                    form.button(`button.edit_locked`)
                } else { form.button(`button.edit`) }
                form_responses.push(r => {
                    editEnderAltar(entity, source)
                })
            }
            {
                form.button(` ${entity.nameTag}`) //ender_altar Name
                form_responses.push(r => {})
            }
            {
                form.button(` ${entity.getTags().find(tag => tag.startsWith('ender_altar_owner:'))?.replace('ender_altar_owner:', '')}`) //ender_altar Owner
                form_responses.push(r => {})
            }
            {
                form.button(` ${x}, ${y}, ${z}`) //ender_altar Position
                form_responses.push(r => {})
            }
            {
                form.button(` ${entity.dimension.id.replaceAll("minecraft:", "")}`) //ender_altar Dimension
                form_responses.push(r => {})
            }
            {
                form.button(` ${entity.getTags().find(tag => tag.startsWith('about_ender_altar:'))?.replace('about_ender_altar:', '')}`) //About ender_altar
                form_responses.push(r => {})
            }
            {
                if (source.getTags().find((tag) => tag.startsWith("ender_altars_tabs:private"))) {
                    form.button(`button.private_ender_altars_tab_locked`)
                } else { form.button(`button.private_ender_altars_tab`) }
                form_responses.push(r => {
                    source.addTag("ender_altars_tabs:private")
                    source.removeTag("ender_altars_tabs:global")
                    ender_altarScreen(source, playerDimension, playerLevel, x, y, z)
                })
            } 
            {
                if (source.getTags().find((tag) => tag.startsWith("ender_altars_tabs:global"))) {
                    form.button(`button.global_ender_altars_tab_locked`)
                } else { form.button(`button.global_ender_altars_tab`) }
                form_responses.push(r => {
                    source.addTag("ender_altars_tabs:global")
                    source.removeTag("ender_altars_tabs:private")
                    ender_altarScreen(source, playerDimension, playerLevel, x, y, z)
                })
            }
        }
    })

    //Overworld ender_altars Buttons
    if (source.hasTag("dimensions_tabs:overworld")) {
        //If the player is on overworld, then display this:
        Array.from(world.getDimension("overworld").getEntities({
            type: "way:ender_altar_dummy",
            tags: [
                "is_global"
            ],
            location: new Location(x, y - 0.5, z),
            minDistance: 1
        })).forEach(entity => {
            if (entity.dimension.id != "minecraft:overworld") return
            const add_dust_icon = []
            if (entity.dimension.id != source.dimension.id) {
                add_dust_icon.push('')
            }
            if (source.hasTag("ender_altars_tabs:global")) {
                let entityDimension = entity.dimension.id
                let entityX = entity.location.x + 1
                let entityY = entity.location.y
                let entityZ = entity.location.z
                form.button(`${add_dust_icon} ${entity.nameTag}`)
                form_responses.push(r => {
                    if (entityDimension != source.dimension.id) {
                        if (playerLevel >= 5) {
                            source.runCommandAsync(`execute in ${entityDimension.replaceAll('minecraft:', '')} run tp ${entityX} ${entityY} ${entityZ}`)
                            source.runCommandAsync(`xp -3l @s`)
                        } else {
                            source.runCommandAsync(`tellraw @s {"rawtext":[{"translate":"text.not_sufficient_xp"}]}`)
                        }
                    } else {
                        source.runCommandAsync(`execute in ${entityDimension.replaceAll('minecraft:', '')} run tp ${entityX} ${entityY} ${entityZ}`)
                    }
                })
            }
        })
        Array.from(world.getDimension("overworld").getEntities({
            type: "way:ender_altar_dummy",
            tags: [
                "not_global"
            ],
            location: new Location(x, y - 0.5, z),
            minDistance: 1
        })).forEach(entity => {
            if (entity.dimension.id != "minecraft:overworld") return
            const add_dust_icon = []
            if (entity.dimension.id != source.dimension.id) {
                add_dust_icon.push('')
            }
            if (source.hasTag("ender_altars_tabs:private") && entity.hasTag(`${source.nameTag}_acess`)) {
                let entityDimension = entity.dimension.id
                let entityX = entity.location.x + 1
                let entityY = entity.location.y
                let entityZ = entity.location.z
                form.button(`${add_dust_icon} ${entity.nameTag}`)
                form_responses.push(r => {
                    if (entityDimension != source.dimension.id) {
                        if (playerLevel >= 5) {
                            source.runCommandAsync(`execute in ${entityDimension.replaceAll('minecraft:', '')} run tp ${entityX} ${entityY} ${entityZ}`)
                            source.runCommandAsync(`xp -3l @s`)
                        } else {
                            source.runCommandAsync(`tellraw @s {"rawtext":[{"translate":"text.not_sufficient_xp"}]}`)
                        }
                    } else {
                        source.runCommandAsync(`execute in ${entityDimension.replaceAll('minecraft:', '')} run tp ${entityX} ${entityY} ${entityZ}`)
                    }
                })
            }
        })
    }
    if (source.hasTag("dimensions_tabs:nether")) {
        //If the player is on nether, then display this:
        Array.from(world.getDimension("nether").getEntities({
            type: "way:ender_altar_dummy",
            tags: [
                "is_global"
            ],
            location: new Location(x, y - 0.5, z),
            minDistance: 1
        })).forEach(entity => {
            if (entity.dimension.id != "minecraft:nether") return
            const add_dust_icon = []
            if (entity.dimension.id != source.dimension.id) {
                add_dust_icon.push('')
            }
            if (source.hasTag("ender_altars_tabs:global")) {
                let entityDimension = entity.dimension.id
                let entityX = entity.location.x + 1
                let entityY = entity.location.y
                let entityZ = entity.location.z
                form.button(`${add_dust_icon} ${entity.nameTag}`)
                form_responses.push(r => {
                    if (entityDimension != source.dimension.id) {
                        if (playerLevel >= 5) {
                            source.runCommandAsync(`execute in nether run tp ${entityX} ${entityY} ${entityZ}`)
                            source.runCommandAsync(`xp -3l @s`)
                        } else {
                            source.runCommandAsync(`tellraw @s {"rawtext":[{"translate":"text.not_sufficient_xp"}]}`)
                        }
                    } else {
                        source.runCommandAsync(`execute in nether run tp ${entityX} ${entityY} ${entityZ}`)
                    }
                })
            }
        })
        Array.from(world.getDimension("nether").getEntities({
            type: "way:ender_altar_dummy",
            tags: [
                "not_global"
            ],
            location: new Location(x, y - 0.5, z),
            minDistance: 1
        })).forEach(entity => {
            if (entity.dimension.id != "minecraft:nether") return
            const add_dust_icon = []
            if (entity.dimension.id != source.dimension.id) {
                add_dust_icon.push('')
            }
            if (source.hasTag("ender_altars_tabs:private") && entity.hasTag(`${source.nameTag}_acess`)) {
                let entityDimension = entity.dimension.id
                let entityX = entity.location.x + 1
                let entityY = entity.location.y
                let entityZ = entity.location.z
                form.button(`${add_dust_icon} ${entity.nameTag}`)
                form_responses.push(r => {
                    if (entityDimension != source.dimension.id) {
                        if (playerLevel >= 5) {
                            source.runCommandAsync(`execute in nether run tp ${entityX} ${entityY} ${entityZ}`)
                            source.runCommandAsync(`xp -3l @s`)
                        } else {
                            source.runCommandAsync(`tellraw @s {"rawtext":[{"translate":"text.not_sufficient_xp"}]}`)
                        }
                    } else {
                        source.runCommandAsync(`execute in nether run tp ${entityX} ${entityY} ${entityZ}`)
                    }
                })
            }
        })
    }
    if (source.hasTag("dimensions_tabs:the_end")) {
        //If the player is on the_end, then display this:
        Array.from(world.getDimension("the_end").getEntities({
            type: "way:ender_altar_dummy",
            tags: [
                "is_global"
            ],
            location: new Location(x, y - 0.5, z),
            minDistance: 1
        })).forEach(entity => {
            if (entity.dimension.id != "minecraft:the_end") return
            const add_dust_icon = []
            if (entity.dimension.id != source.dimension.id) {
                add_dust_icon.push('')
            }
            if (source.hasTag("ender_altars_tabs:global")) {
                let entityDimension = entity.dimension.id
                let entityX = entity.location.x + 1
                let entityY = entity.location.y
                let entityZ = entity.location.z
                form.button(`${add_dust_icon} ${entity.nameTag}`)
                form_responses.push(r => {
                    if (entityDimension != source.dimension.id) {
                        if (playerLevel >= 5) {
                            source.runCommandAsync(`execute in the_end run tp ${entityX} ${entityY} ${entityZ}`)
                            source.runCommandAsync(`xp -3l @s`)
                        } else {
                            source.runCommandAsync(`tellraw @s {"rawtext":[{"translate":"text.not_sufficient_xp"}]}`)
                        }
                    } else {
                        source.runCommandAsync(`execute in the_end run tp ${entityX} ${entityY} ${entityZ}`)
                    }
                })
            }
        })
        Array.from(world.getDimension("the_end").getEntities({
            type: "way:ender_altar_dummy",
            tags: [
                "not_global"
            ],
            location: new Location(x, y - 0.5, z),
            minDistance: 1
        })).forEach(entity => {
            if (entity.dimension.id != "minecraft:the_end") return
            const add_dust_icon = []
            if (entity.dimension.id != source.dimension.id) {
                add_dust_icon.push('')
            }
            if (source.hasTag("ender_altars_tabs:private") && entity.hasTag(`${source.nameTag}_acess`)) {
                let entityDimension = entity.dimension.id
                let entityX = entity.location.x + 1
                let entityY = entity.location.y
                let entityZ = entity.location.z
                form.button(`${add_dust_icon} ${entity.nameTag}`)
                form_responses.push(r => {
                    if (entityDimension != source.dimension.id) {
                        if (playerLevel >= 5) {
                            source.runCommandAsync(`execute in the_end run tp ${entityX} ${entityY} ${entityZ}`)
                            source.runCommandAsync(`xp -3l @s`)
                        } else {
                            source.runCommandAsync(`tellraw @s {"rawtext":[{"translate":"text.not_sufficient_xp"}]}`)
                        }
                    } else {
                        source.runCommandAsync(`execute in the_end run tp ${entityX} ${entityY} ${entityZ}`)
                    }
                })
            }
        })
    }
    form.show(source).then(r => {
        form_responses[r.selection]()
    })
}