import {
    world,
    BlockLocation,
    Location,
    EntityQueryOptions
} from "@minecraft/server"
import {
    ActionFormData,
    ModalFormData
} from '@minecraft/server-ui'
import {
    getPlayerExperienceLevel
} from "./functions"
import {
    ender_altarScreen
} from "./forms"

world.events.blockPlace.subscribe(data => {
    const { block, dimension, player } = data
    const { location } = block
    const { x, y, z } = location
    const blockAboveLocation = new BlockLocation(x, y + 1, z)
    const blockAbove = world.getDimension(`${dimension.id.replaceAll('minecraft:', '')}`).getBlock(blockAboveLocation)

    if (block.hasTag('ender_altar')) {

        //If block above doesn't equals air, execute this:
        if (blockAbove.typeId != 'minecraft:air') {
            dimension.runCommandAsync(`setblock ${x} ${y} ${z} air`)
            dimension.runCommandAsync(`gamerule sendcommandfeedback false`)
            player.runCommandAsync(`give @s[m=!creative] ${block.typeId}`)
            dimension.runCommandAsync(`gamerule sendcommandfeedback true`)
        } /* If block above equals air, then execute this: */ else {
            //Place Top Block of the ender_altar
            dimension.runCommandAsync(`setblock ${x} ${y + 1} ${z} way:ender_altars_top_block`)

            //Form
            const form = new ModalFormData
            form.title(`tile.${block.typeId}.name`)
            form.textField("text.enter_ender_altar_name", "")
            form.textField("text.enter_ender_altar_description", "")
            form.toggle('toggle.is_global', false)
            form.show(player).then((r) => {

                player.addTag("dimensions_tabs:overworld")
                player.removeTag("dimensions_tabs:nether")
                player.removeTag("dimensions_tabs:the_end")
                player.addTag("ender_altars_tabs:private")
                player.removeTag("ender_altars_tabs:global")

                //Activating the ender_altar


                // Getting the blocks permutations
                let blockPermutation = block.permutation
                let blockAbovePermutation = blockAbove.permutation

                // Set the values
                blockPermutation.getProperty("property:active").value = true
                blockAbovePermutation.getProperty("property:active").value = true

                // Set back to blocks
                block.setPermutation(blockPermutation)
                blockAbove.setPermutation(blockAbovePermutation)


                //Form Results
                const { canceled, formValues } = r

                //If form is canceled then set name unregistered and make it not global
                if (canceled === true) {
                    try {
                        const entity = world.getDimension(`${dimension.id.replaceAll('minecraft:', '')}`).spawnEntity("way:ender_altar_dummy", new Location(x + 0.5, y, z + 0.5))
                        entity.nameTag = `name.unregistered`
                        entity.addTag(`not_global`)
                        entity.addTag(`${player.nameTag}_acess`)
                        entity.addTag(`entity_id:${entity.id}`)
                        entity.addTag(`about_ender_altar:`)
                        entity.addTag(`ender_altar_owner:${player.nameTag}`)
                    } catch { }
                }
                //TextField (Name of the ender_altar)
                if (formValues[0] != '') {
                    try {
                        const entity = world.getDimension(`${dimension.id.replaceAll('minecraft:', '')}`).spawnEntity("way:ender_altar_dummy", new Location(x + 0.5, y, z + 0.5))
                        entity.nameTag = `${formValues[0]}`

                        //Add Description
                        if (formValues[1] != '') {
                            entity.addTag(`about_ender_altar:${formValues[1]}`)
                        }
                        entity.addTag(`ender_altar_owner:${player.nameTag}`)


                        //Toggle (Is global? Yes or no?)
                        if (formValues[2] == true) {
                            try {
                                entity.addTag(`is_global`)
                                entity.addTag(`entity_id:${entity.id}`)
                            } catch { }
                        } else {
                            entity.addTag(`not_global`)
                            entity.addTag(`${player.nameTag}_acess`)
                            entity.addTag(`entity_id:${entity.id}`)
                        }
                    } catch { }
                } else {
                    try {
                        const entity = world.getDimension(`${dimension.id.replaceAll('minecraft:', '')}`).spawnEntity("way:ender_altar_dummy", new Location(x + 0.5, y, z + 0.5))
                        entity.nameTag = `name.unregistered`

                        //Add Description
                        if (formValues[1] != '') {
                            entity.addTag(`about_ender_altar:${formValues[1]}`)
                        }
                        entity.addTag(`ender_altar_owner:${player.nameTag}`)

                        //Toggle (Is global? Yes or no?)
                        if (formValues[2] == true) {
                            try {
                                entity.addTag(`is_global`)
                                entity.addTag(`entity_id:${entity.id}`)
                            } catch { }
                        } else {
                            entity.addTag(`not_global`)
                            entity.addTag(`${player.nameTag}_acess`)
                            entity.addTag(`entity_id:${entity.id}`)
                        }
                    } catch { }
                }
            })
        }
    }
})


world.events.beforeItemUseOn.subscribe(eventData => {
    const { blockLocation, source } = eventData

    const playerLevel = getPlayerExperienceLevel(source)
    const playerDimension = []
    switch (source.dimension.id) {
        case "minecraft:overworld":
            playerDimension.push('overworld')
            break
        case "minecraft:nether":
            playerDimension.push('nether')
            break
        case "minecraft:the_end":
            playerDimension.push('the_end')
            break
    }

    const { x, y, z } = blockLocation
    const blocklocation = new BlockLocation(x, y, z)
    const block = world.getDimension(`${playerDimension}`).getBlock(blocklocation)

    try {
        if (block.hasTag('ender_altar') && block.permutation.getProperty('property:active').value == true && !source.isSneaking) {
            ender_altarScreen(source, playerDimension, playerLevel, x, y, z)
        }
    } catch { }
})